/*
    opinionDynamics 0.3: ExperSwarm.m
    Copyright (C) 2005 by Gary McKeown
	
	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

    Contact information:
		Gary McKeown,
		School of Psychology,
		Queens University of Belfast,
		Belfast BT7 1NN,
		Northern Ireland, 
		UK.
	g.mckeown@qub.ac.uk
*/

#import "ExperSwarm.h"
#import "BatchSwarm.h"
#import <simtoolsgui.h>
#import <defobj.h> // Archivers
#include <misc.h> // unlink
#import "Parameters.h"

@implementation ExperSwarm

+ createBegin: aZone
{
	ExperSwarm *obj;
 
	obj = [super createBegin: aZone];
	obj->numModelsRun = 0;
	
	// initialize parameters
	[(Parameters *)arguments setThreshold:0.00];
	[(Parameters *)arguments setPolarisation:1];
	[(Parameters *)arguments setBroadcastRatio:0];

	return obj;
}

- buildObjects
{

	[super buildObjects];
//	FILE *file1;
//	if ((file1 = fopen("~/resultsStdDev.txt", "a")) == NULL) {
//		fprintf(stderr, "Cannot open %s\n", "file");
//	} 
	[self prepareCOutputFile];
	return self;
}

- buildActions
{
	[super buildActions];
	experActions = [ActionGroup create: self];

	[experActions createActionTo:self message: M(buildModel)];
	[experActions createActionTo:self message: M(runModel)];
	[experActions createActionTo: self message: M(doStats)];
	[experActions createActionTo:self message: M(dropModel)];

	[experActions createActionTo:self message: M(checkToStop)];
  
	experSchedule = [Schedule createBegin: self];
	[experSchedule setRepeatInterval: 1];
	experSchedule = [experSchedule createEnd];
	[experSchedule at: 0 createAction: experActions];

	return self;
}  



- activateIn: swarmContext
{
	[super activateIn: swarmContext];
	[experSchedule activateIn: self];
	return [self getActivity];
}

- buildModel
{
	modelSwarm = [ModelSwarm create: self];
  
	[modelSwarm buildObjects];
	[modelSwarm buildActions];
	[modelSwarm activateIn: nil];

	return self;
}  


- runModel
{
	[[modelSwarm getActivity] run];
	numModelsRun++;               // increment count of models

//	fprintf(stderr, "end of runModel\n");
	[[modelSwarm getActivity] stop];
	return self;
}

- doStats
{
	[self writeCData];
	printf ("That was iteration %lu, stdDev =%lf, opinion =%lf, threshold =%lf, polarisation =%lf\n", getCurrentTime(), [modelSwarm getStandardDeviation], [modelSwarm getMeanOpinion], [(Parameters *)arguments threshold], [(Parameters *)arguments polarisation]);//[(Parameters *)arguments broadcastRatio],broadcastRatio = %d,
	printf ("broadcastRatio = %d\n", [(Parameters *)arguments broadcastRatio]);
	// put carriage return at the end of the data file after 101 iterations.
	if ((numModelsRun % 101) == 0) { 
		fprintf (dataExperOutputFileStdDev, "\n");
		fflush ( dataExperOutputFileStdDev );
	
		fprintf (dataExperOutputFileOpinion, "\n");
		fflush ( dataExperOutputFileOpinion );
		
		fprintf (dataExperOutputFileIndicator, "\n");
		fflush ( dataExperOutputFileIndicator );
	}
	return self;
}

- dropModel
{
	[modelSwarm drop];
	return self;
}

- checkToStop
{
	if (!([self stepParameters]))
		[self stopRunning];
	return self;
}

- stopRunning
{
	[getTopLevelActivity() terminate]; // Terminate the simulation.
	printf("\n All the models have run!\n");
	return self;
}


- go 
{
	[[self getActivity] run];
	return [[self getActivity] getStatus];
}


- (BOOL)stepParameters
{

	[(Parameters *)arguments incrementThreshold];
	if ([(Parameters *)arguments threshold] > 2.01) {
//		[(Parameters *)arguments incrementPolarisation];  // change here for polarisation parameter sweep
		[(Parameters *)arguments incrementBroadcastRatio];
		[(Parameters *)arguments setThreshold:0.00];
	}
	if (([(Parameters *)arguments threshold] > 2.03) || ([(Parameters *)arguments broadcastRatio] > 100))//([(Parameters *)arguments polarisation] > 1.01)) // change this for polarisation parameter sweep
		return NO;

	return YES;
}

- prepareCOutputFile
{
	char outputFileStdDev[256];
	char outputFileOpinion[256];
	char outputFileIndicator[256];

	if (dataFileExists == YES) return self;
	else {
			snprintf (outputFileStdDev,256, "stdDev.data");
			snprintf (outputFileOpinion,256, "opinions.data");
			snprintf (outputFileIndicator,256, "indicator.data");
		if(!(dataExperOutputFileStdDev = fopen(outputFileStdDev,"w")))
			{ abort(); printf ("failed to open dataExperOutputFileStdDev\n"); }
		if(!(dataExperOutputFileOpinion = fopen(outputFileOpinion,"w")))
			{ abort(); printf ("failed to open dataExperOutputFileOpinion\n"); }
		if(!(dataExperOutputFileIndicator = fopen(outputFileIndicator,"w")))
			{ abort(); printf ("failed to open dataExperOutputFileIndicator\n"); }
//		fprintf (dataExperOutputFileStdDev, "currentTime\t avgOpinion\t Standard Deviation\n");
		dataFileExists = YES;
	}
	return self;
}

- writeCData
{
	// The \t is for tab separated data
	fprintf (dataExperOutputFileStdDev, "%lf\t", [modelSwarm getStandardDeviation]);
	fflush ( dataExperOutputFileStdDev );
	
	fprintf (dataExperOutputFileOpinion, "%lf\t", [modelSwarm getMeanOpinion]);
	fflush ( dataExperOutputFileOpinion );

	fprintf (dataExperOutputFileIndicator, "%lf\t", [modelSwarm getIndicator]);
	fflush ( dataExperOutputFileIndicator );
	return self;
}

- (void)drop
{
	if (dataExperOutputFileStdDev) fclose(dataExperOutputFileStdDev);
	if (dataExperOutputFileOpinion) fclose(dataExperOutputFileOpinion);
	if (dataExperOutputFileIndicator) fclose(dataExperOutputFileIndicator);
	[super drop];
}

@end
