/*
    opinionDynamics 0.3: ModelSwarm.m
    Copyright (C) 2005 by Gary McKeown
	
	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

    Contact information:
		Gary McKeown,
		School of Psychology,
		Queens University of Belfast,
		Belfast BT7 1NN,
		Northern Ireland, 
		UK.
	g.mckeown@qub.ac.uk
*/

#import "ModelSwarm.h"
#import "Parameters.h"
#import "Person.h"
#import <simtools.h>
#import <analysis.h>

@implementation ModelSwarm


- createEnd
{
	return [super createEnd];
}

- buildObjects 
{
	int x,y;
	int numAgents = 1;
//	int numsampled = 0;

	[super buildObjects];

	agentList = [List create: self];  // the List of agents
	sampleList= [List create: self];  // the List of sampled agents
	
//	aNormalDist = [NormalDist createWithDefaults: [self getZone]];
	
	//Create the world
	world = [OpinionWorld createBegin: self ];
	[world setSizeX:[(Parameters *)arguments worldSize] Y:[(Parameters *)arguments worldSize]];
	[world setModelSwarm: self];
	world = [world createEnd];

	randomBit = [RandomBitDist create: self setGenerator: randomGenerator];

	// Create the Agents
	for (x = 0; x < [(Parameters *)arguments worldSize]; x++) 
    {
		for(y = 0; y < [(Parameters *)arguments worldSize]; y++) 
		{
			Person *person;
			// Create an actual agent/person
			person=[Person createBegin: self];
			[person setWorld: world];
			person = [person createEnd];
			[person setX: x Y: y];

			[person setIDNumber: numAgents];
			numAgents++;

			[person setPersonalConvergence:[(Parameters *)arguments convergence]];
			[person setPersonalThreshold:[(Parameters *)arguments threshold]];
//			[person setOpinion:[aNormalDist getSampleWithMean:0 withStdDev:0.3]];
			[person setOpinion:[uniformDblRand getDoubleWithMin:-1.0 withMax:1.0]];
			if (swarmGUIMode == YES) {
				int opinionColor = ([person getOpinion]*100) + 100;
				[person setColor:opinionColor];
			}
			// Add person to world and Agentlist
			[world addObject: person atX: x Y: y];
			[agentList addLast: person];	
		}
    }
	
	// Sample n agents
	[NSelect select:100 from:agentList into:sampleList];
		
	// Create the opinion Averager to get stats
	opinionAverager = [Averager createBegin:globalZone];
	[opinionAverager setCollection:agentList];
	[opinionAverager setProbedSelector: M(getOpinion)];
	opinionAverager = [opinionAverager createEnd];
	
	//output file
	[self prepareCDataFile];

	return self;
}


- getAgentList 
{
	return agentList;
}

- getWorld 
{
	return world;
}

- (double)getStandardDeviation
{
	[opinionAverager update];
	double stdDev = [opinionAverager getStdDev];
	return stdDev;
}

- (double)getMeanOpinion
{
	[opinionAverager update];
	double opinion = [opinionAverager getAverage];
	return opinion;
}

- (double)getIndicator
{
	id <Index> index;
	Person *anObject;
	double lastOpinion = 0;
	int numberOfAgents = [agentList getCount];
	int k = 0; // number of clusters
	int count = 0, total=0, debugcount = 0;
	double y = 0, p, psquared, absolutex, party;
	[QSort sortObjectsIn:agentList];
	index = [agentList begin: self];
	for (anObject = [index next]; [index getLoc]==Member; anObject = [index next])
	{
		debugcount++;
		if (k == 0) {
			k = 1;
			total = 1;
			count = 1;
			lastOpinion = [anObject getRoundedOpinion];
		} else {
			if (([anObject getRoundedOpinion] == lastOpinion)) {
				count++;
				total++;
				lastOpinion = [anObject getRoundedOpinion];
			} else {
				p = (((double)count)/numberOfAgents);
				psquared = pow(p,2);
				absolutex = fabs(lastOpinion);
				party = psquared * absolutex;
				y += party;
				lastOpinion = [anObject getRoundedOpinion];
				k++;
				count = 1;
				total++;
			} 
		}
		if (total == numberOfAgents) {
				p = (((double)count)/numberOfAgents);
				psquared = pow(p,2);
				absolutex = fabs(lastOpinion);
				party = psquared * absolutex;				
				y += party;
		}
	}
	
	return y;
}
		
- buildActions 
{
	[super buildActions];
  
	modelActions = [ActionGroup create: self];
	[modelActions createActionTo: self message: M(stepThroughList)];
	[modelActions createActionTo: self message: M(checkToStop)];
      
	modelSchedule = [Schedule createBegin: self];
	[modelSchedule setRepeatInterval: 1];
	modelSchedule = [modelSchedule createEnd];
	[modelSchedule at: 0 createAction: modelActions]; 

	return self;
}


- stepThroughList
{
	id <Index> index;
	id <Index> sampleIndex;
	id anObject;
	id sampledObject;
	
	if ([(Parameters *)arguments randomize] == YES)
	{
		id shuffler = [ListShuffler create: self];
		[shuffler shuffleWholeList: agentList];
		[shuffler drop];
	}

	index = [agentList begin: self];
//	int opinionDynamics 0.3:0, maxcount=0;
	for (anObject = [index next]; [index getLoc]==Member; anObject = [index next])
	{
		if (([(Parameters *)arguments polarisationPresent]) && ([uniformUnsRand getUnsignedWithMin:0 withMax:[(Parameters *)arguments broadcastRatio]] == 1)) {
			if ([randomBit getCoinToss]) {
				[anObject stepBroadcast:[(Parameters *)arguments minOpinion]];
//				mincount++;
//				printf("min\n");
			} else {
				[anObject stepBroadcast:[(Parameters *)arguments maxOpinion]];
//				maxcount++;
//				printf("max\n");
			}			
		} else {
			[anObject step];
		}
	}
	if([(Parameters *)arguments collectData]) { 
			sampleIndex = [sampleList begin: self];
			for (sampledObject = [sampleIndex next]; [sampleIndex getLoc]==Member; sampledObject = [sampleIndex next])
			{
				[self writeStepData:[sampledObject getOpinion]];
			}
	}
	[self listEnd];
	[index drop];
//	[self checkToStop];
//	printf("mincount=%d, maxcount=%d\n", mincount, maxcount);
	return self;
}

- (BOOL)randomizeList
{
  if ([(Parameters *)arguments randomize] == YES) [(Parameters *)arguments setRandomize:NO];
  else [(Parameters *)arguments setRandomize:YES];
  return [(Parameters *)arguments randomize];
}

- activateIn: swarmContext 
{
  [super activateIn: swarmContext];

  [modelSchedule activateIn: self];

  return [self getSwarmActivity];
}

- checkToStop
{
	long currentTime = getCurrentTime();
	if (currentTime == getInt ((Parameters *)arguments, "experimentDuration"))
    {
//		[self getIndicator];
		[self stopRunning];
    } 
	return self;
}

- prepareCDataFile
{
// uncomment below to collect opinionData for a sample of the agents in the list

	char outputFileOpinion[256];

	if (dataFileExists == YES) return self;
		else{
			snprintf (outputFileOpinion,256, "opinionCollection.data");
		if(!(dataObserverOutputFileOpinion = fopen(outputFileOpinion,"w")))
			abort();
		dataFileExists = YES;
	}
	return self;
}

- writeStepData:(double)opinion
{
	fprintf (dataObserverOutputFileOpinion, "%lf\t", opinion);
	fflush (dataObserverOutputFileOpinion );
	return self;
}
- listEnd
{
	fprintf (dataObserverOutputFileOpinion, "\n");
	fflush (dataObserverOutputFileOpinion );
	return self;
}

- doEndStats
{
	return self;
}

- stopRunning
{
//	[self doEndStats];
	[getTopLevelActivity() stop]; // Terminate the simulation.
	return self;
}

- (void)drop
{
	if (dataObserverOutputFileOpinion) fclose(dataObserverOutputFileOpinion);
	[super drop];
}


@end








