/*
    opinionDynamics 0.3: Parameters.m
    Copyright (C) 2005 by Gary McKeown
	
	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

    Contact information:
		Gary McKeown,
		School of Psychology,
		Queens University of Belfast,
		Belfast BT7 1NN,
		Northern Ireland, 
		UK.
	g.mckeown@qub.ac.uk
*/

#import <objectbase.h>
#import "Parameters.h"
#import <misc.h>  

id makeProbe (id obj, const char *ivarName)
{
	id probe = [VarProbe createBegin: [obj getZone]];
	[probe setProbedClass: [obj getClass]];
	[probe setProbedVariable: ivarName];
	return [probe createEnd];
}

int
getInt (id obj, const char *ivarName)
{
	id probe = makeProbe (obj, ivarName);
	int ret = [probe probeAsInt: obj];
	[probe drop];
	return ret;
}

@implementation Parameters

+ createBegin: aZone
 {
	static struct argp_option options[] = {
		 {"inputfile", 'I',"filename",0,"set fn",5},
		 {"run",'R',"RunNumber",0,"Run is...",6},
		 {"threshold",'h',"T", 0, "Threshold", 8},
		 {"convergence",'c',"C",0,"Convergence",9},
		 {"polarisation",'p',"P",0,"Degree of Polarisation",10},
		 {"broadcastRatio",'r',"ratio",0,"Ratio of social interactions for every broadcast interaction",11},	
		 {"worldSize",'x',"X",0,"Size of X dimension",12},
		 {"experimentDuration",'e',"E",0,"Duration of experiment",13},
		 {"seed",'S',"SeedNumber",0,"Seed",14},
		 { 0 }
   };
   
	Parameters *obj;
	obj = [super createBegin: aZone];
	[obj addOptions: options];

	// default parameters values.
	obj->worldSize = 50;
	obj->threshold = 1;
	obj->convergence = 0.3;
	obj->broadcastRatio = 5;
	obj->polarisation = 1;
	obj->minOpinion = -1.0;
	obj->maxOpinion = 1.0;
	obj->randomize = NO; // change this to randomize the agent list before an iteration creates a performance hit.
	obj->experimentDuration = 0; // change this to set length of experiment 1 = an iteration for each agent
	obj->thresholdInc = 0.02; // for experiment/batch runs
	obj->polarisationInc = 0.01; // for experiment/batch runs
	obj->broadcastRatioInc = 1; // for experiment/batch runs
	obj->polarisationPresent = YES;
	obj->collectData = YES; // for turns on data collection of sampled agents
	
	return obj;
}

- init
{
	id <ProbeMap> probeMap;
	probeMap = [EmptyProbeMap createBegin:[self getZone]];
	[probeMap setProbedClass: [self class]];
	probeMap = [probeMap createEnd];

	// probes for simulation parameters
	[probeMap addProbe: [probeLibrary getProbeForVariable: "worldSize"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "convergence"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "threshold"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "broadcastRatio"
										inClass: [self class]]];
//	[probeMap addProbe: [probeLibrary getProbeForVariable: "polarisation"
//										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "polarisationPresent"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "maxOpinion"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "minOpinion"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "randomSeed"
										inClass: [self class]]];
	[probeMap addProbe: [probeLibrary getProbeForVariable: "experimentDuration"
										inClass: [self class]]];

	[probeLibrary setProbeMap: probeMap For: [self class]];

	return self;
}



- (int)parseKey: (int) key arg: (const char*) arg
{
  switch ( key )
    {
    case  'h' :
		if (arg)
		threshold = strtod (arg,NULL);  
		break;
    
    case 'c' :
		if (arg)
		convergence = strtod (arg,NULL);
		break;
	
	case  'p' :
		if (arg)
       polarisation = strtod (arg,NULL);
		break;
	
	case  'r' :
		if (arg)
        broadcastRatio = strtod (arg,NULL);
		break;

    case 'x' :
		if (arg)
		worldSize = atoi(arg);   //atoi--ascii to integer from stdlib
		break;
         
    case 'e' :
		if (arg)
		experimentDuration = atoi (arg);
		break;
    
    case 'I' :
		printf("here is an I. You might need it someday, drone reasons\n");
		break;
	
	case 'S' :
		printf("here is an S. You might need it someday, drone reasons\n");
		break;
     
    case  'R' :
		printf("here is an R. You might need it someday, drone reasons\n");
//		if (arg)
//		run = atoi(arg);
		break;
     
    default : 
      return [super parseKey: key arg: arg];
    }
  return 0;
}

- (double)threshold
{
	return threshold;
}
- (void)setThreshold:(double)newThreshold
{
	threshold = newThreshold;
}

- (double)convergence
{
	return convergence;
}

- (double)polarisation
{
	return polarisation;
}
- (void)setPolarisation:(double)newPolarisation
{
	polarisation = newPolarisation;
	[self setMaxOpinion:(0 + [(Parameters *)arguments polarisation])];
	[self setMinOpinion:(0 - [(Parameters *)arguments polarisation])];
}
- (void)setBroadcastRatio:(int)newBroadcastRatio
{
	broadcastRatio = newBroadcastRatio;
}
- (int)broadcastRatio
{
	return broadcastRatio;
}
- (double)thresholdInc
{
	return thresholdInc;
}
- (void)setThresholdInc:(double)newThresholdInc
{
	thresholdInc = newThresholdInc;
}
- (double)polarisationInc
{
	return polarisationInc;
}
- (void)setPolarisationInc:(double)newPolarisationInc
{
	polarisationInc = newPolarisationInc;
}
- (int)broadcastRatioInc
{
	return broadcastRatioInc;
}
- (void)setBroadcastRatioInc:(int)newBroadcastRatioInc
{
	broadcastRatioInc = newBroadcastRatioInc;
}
- (BOOL)polarisationPresent
{
	return polarisationPresent;
}
- (void)setPolarisationPresent:(BOOL)newPolarisationPresent
{
	polarisationPresent = newPolarisationPresent;
}

- (double)minOpinion
{
	return minOpinion;
}
- (void)setMinOpinion:(double)newMinOpinion
{
	minOpinion = newMinOpinion;
}
- (double)maxOpinion
{
	return maxOpinion;
}
- (void)setMaxOpinion:(double)newMaxOpinion
{
	maxOpinion = newMaxOpinion;
}


- (int)worldSize
{
	return worldSize;
}

- (BOOL)randomize
{
	return randomize;
}
- (BOOL)collectData
{
	return collectData;
}

- (void)setRandomize:(BOOL)newRandomize
{
	randomize = newRandomize;
}

- (void)setCurrentTime: (long)x
{
	currentTime=x;
}

- (timeval_t)getCurrentTime
{
	return currentTime;
}

- (void)incrementThreshold
{
	threshold += thresholdInc;
}
- (void)incrementPolarisation
{
	[self setPolarisation:polarisation +=polarisationInc];
}
- (void)incrementBroadcastRatio
{
	[self setBroadcastRatio:broadcastRatio +=broadcastRatioInc];
}



@end



