/*
    opinionDynamics 0.3: Person.m
    Copyright (C) 2005 by Gary McKeown
	
	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

    Contact information:
		Gary McKeown,
		School of Psychology,
		Queens University of Belfast,
		Belfast BT7 1NN,
		Northern Ireland, 
		UK.
	g.mckeown@qub.ac.uk
*/

#import "Person.h"
#import "ModelSwarm.h"
#import "Parameters.h"
#import <simtools.h>

@implementation Person

- createEnd
{
  return [super createEnd];
}

// Accessor methods
- setX: (int)inx Y: (int)iny 
{
  x = inx;
  y = iny;
  
  return self;
}

- (int)getX 
{
  return x;
}

- (int)getY 
{
  return y;
}

- (void)setPersonalConvergence: (double)convergence
{
	personalConvergence = convergence;
}

- (void)setPersonalThreshold: (double)threshold
{
	personalThreshold = threshold;
}

- setWorld: w 
{
  myWorld = w;
  xsize = [[myWorld getObjectGrid] getSizeX];
  ysize = [[myWorld getObjectGrid] getSizeY];

  return self;
}

- setColor: (int)c 
{
  myColor = c;
  return self;
}

- (int)getColor 
{
  return myColor;
}

- (void)setIDNumber: (int)numb
{
  idNumber = numb;
}	

- (int)idNumber
{
	return idNumber;
}

- (double)getOpinion 
{
  return opinion;
}
- setOpinion: (double)newOpinion
{
  opinion = newOpinion;  
  return self;
}

- (double)getRoundedOpinion
{
	double rounded = round(([self getOpinion] * 1000));
//	printf("\nrounded = %lf\n return = %lf", rounded, (rounded / 1000));
	return (rounded / 1000);
}

// Agent social network behaviour
- step  
{
	//pick a random neighbour & find its opinion
	Person *neighbourAgent = [self getNeighbour];
	double neighbourOpinion = [neighbourAgent getOpinion];
	double ownOpinion = [self getOpinion];
	double difference = fabs((ownOpinion-neighbourOpinion));
	if (difference < personalThreshold) {
		[self setOpinion:(ownOpinion + (personalConvergence*(neighbourOpinion - ownOpinion)))];
		[neighbourAgent setOpinion:(neighbourOpinion + (personalConvergence*(ownOpinion - neighbourOpinion)))];
	}
	
	if (swarmGUIMode == YES) {
		int opinionColor = ([self getOpinion]*100);
		opinionColor = opinionColor + 100;
		[self setColor:opinionColor];
		opinionColor = ([neighbourAgent getOpinion]*100);
		opinionColor = opinionColor + 100;
		[neighbourAgent setColor:opinionColor];
	}
	return self;
}

- stepBroadcast:(double)broadcastOpinion
{
	// conduct a broadcastOpinion interaction
//	double difference = fabs((ownOpinion - broadcastOpinion));
//	if(difference < personalThreshold) {
//		[self setOpinion:(ownOpinion - (personalConvergence * difference))];
//	}
	double ownOpinion = [self getOpinion];
	double difference = ownOpinion - broadcastOpinion;
	if((difference < personalThreshold) && (difference > 0)) { //difference is positive ie mine bigger than broadcast
//		printf("pos old opinion = %lf new opinion =%lf difference = %lf personalThreshold= %lf\n", ownOpinion, (ownOpinion - (personalConvergence * difference)), difference, personalThreshold);
		[self setOpinion:(ownOpinion - (personalConvergence * difference))];
	} else if ((difference > (-personalThreshold)) && (difference < 0)) { //difference is negative ie broadcast bigger than mine
//		printf("neg old opinion = %lf new opinion =%lf difference = %lf personalThreshold= %lf\n", ownOpinion, (ownOpinion - (personalConvergence * difference)), difference, personalThreshold);
		[self setOpinion:(ownOpinion - (personalConvergence * difference))];
	} 
	
	if (swarmGUIMode == YES) {
		int opinionColor = ([self getOpinion]*100);
		opinionColor = opinionColor + 100;
		[self setColor:opinionColor];
	}
	return self;
}

- (Person *)getNeighbour
{
	int neighbourX = 0, neighbourY = 0;
	int neighbour = [uniformUnsRand getUnsignedWithMin:0 withMax:3]; // change max to 7 for a Moore neighbourhood (3 for a Neumann)
	switch ( neighbour )
    {
		case 0:
			neighbourX=[self getX] + 0;
			neighbourY=[self getY] + 1;
			break;
		case 1:
			neighbourX=[self getX] + 1;
			neighbourY=[self getY] + 0;
			break;
		case 2:
			neighbourX=[self getX] + 0;
			neighbourY=[self getY] - 1;
			break;
		case 3:
			neighbourX=[self getX] - 1;
			neighbourY=[self getY] + 0;
			break;
		case 4:
			neighbourX=[self getX] - 1;
			neighbourY=[self getY] + 1;
			break;
		case 5:
			neighbourX=[self getX] + 1;
			neighbourY=[self getY] + 1;
			break;
		case 6:
			neighbourX=[self getX] + 1;
			neighbourY=[self getY] - 1;
			break;
		case 7:
			neighbourX=[self getX] - 1;
			neighbourY=[self getY] - 1;
			break;
	}
//	printf("neighbourX = %d neighbourY = %d neighbour=%d\n", neighbourX, neighbourY, neighbour);
	neighbourX = (neighbourX + xsize) % xsize;
	neighbourY = (neighbourY + ysize) % ysize;
	return [myWorld returnObjectAtX:neighbourX Y:neighbourY];//[myWorld getOpinionAtX:neighbourX Y:neighbourY];
}

- drawSelfOn: (id)rast 
{
  // A method to draw the agent on the raster
  [rast drawPointX: x Y: y Color: myColor];
  return self;
}

- (int)wrapCoord: (int)inCoord atModulus: (int)inModulus
{
  if (inCoord < 0 || inCoord >= inModulus)
    { 
      inCoord = (inCoord + inModulus) % inModulus; 
    }
  return inCoord;

}

- (int)wrapXCoord: (int)inCoord
{
  return [self wrapCoord: inCoord atModulus: xsize];
}

- (int)wrapYCoord: (int)inCoord
{
  return [self wrapCoord: inCoord atModulus: ysize];
}

- (int)compare:(Person *)otherPerson
{
	double otherOpinion = [otherPerson getOpinion];
	return [self getOpinion] < otherOpinion ? -1 : [self getOpinion] != otherOpinion;
}

- (int)compare:(Person *)otherPerson withRange:(double)range
{
	double otherOpinion = [otherPerson getOpinion];
	double myOpinion = [self getOpinion];
	
	if ( myOpinion < (otherOpinion - range) )	{ return -1; }
	else if ( myOpinion > (otherOpinion + range) ) { return 1; }
	else { return 0; }
}


@end
